﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices; // for GetLastError()

using AIOETHW; // the namespace exposes the AIOETH Class interface 

// You must also add a reference to the AIOETHHWNet.dll in the project settings


namespace Sample1
{
    public partial class Form1 : Form
    {
        public UIntPtr uClientRef;
        public Boolean bConnected = false;
        public int pDataBytes = 6;
        public Byte[] pData = new byte[] { 0, 0, 0, 0, 0, 0 };

        public UInt32 Status;

        public UInt32 numPins = 48;
        public UInt32 numPorts = 6;

        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();

        // Array of controls for GUI updates:
        public CheckBox[] bitLevels = new CheckBox[48];
        public Button[] IoButtons = new Button[6];

        // For bitmasks:
        byte[] OutMask = new byte[] { 0, 0 };

        // State flags:
        public bool[] bIsInput = new bool[6];
        
        public Form1()
        {
            InitializeComponent();

            // Array of the input vs output toggle mode buttons:
            IoButtons[0] = btnToggle0;
            IoButtons[1] = btnToggle1;
            IoButtons[2] = btnToggle2;
            IoButtons[3] = btnToggle3;
            IoButtons[4] = btnToggle4;
            IoButtons[5] = btnToggle5;

            // Array of the LED style high/low pin level checkboxes:
            bitLevels[0] = checkBoxA0_1;
            bitLevels[1] = checkBoxA0_2;
            bitLevels[2] = checkBoxA0_3;
            bitLevels[3] = checkBoxA0_4;
            bitLevels[4] = checkBoxA0_5;
            bitLevels[5] = checkBoxA0_6;
            bitLevels[6] = checkBoxA0_7;
            bitLevels[7] = checkBoxA0_8;
            bitLevels[8] = checkBoxB0_9;
            bitLevels[9] = checkBoxB0_10;
            bitLevels[10] = checkBoxB0_11;
            bitLevels[11] = checkBoxB0_12;
            bitLevels[12] = checkBoxB0_13;
            bitLevels[13] = checkBoxB0_14;
            bitLevels[14] = checkBoxB0_15;
            bitLevels[15] = checkBoxB0_16;
            bitLevels[16] = checkBoxC0_17;
            bitLevels[17] = checkBoxC0_18;
            bitLevels[18] = checkBoxC0_19;
            bitLevels[19] = checkBoxC0_20;
            bitLevels[20] = checkBoxC0_21;
            bitLevels[21] = checkBoxC0_22;
            bitLevels[22] = checkBoxC0_23;
            bitLevels[23] = checkBoxC0_24;

            bitLevels[24] = checkBoxA1_25;
            bitLevels[25] = checkBoxA1_26;
            bitLevels[26] = checkBoxA1_27;
            bitLevels[27] = checkBoxA1_28;
            bitLevels[28] = checkBoxA1_29;
            bitLevels[29] = checkBoxA1_30;
            bitLevels[30] = checkBoxA1_31;
            bitLevels[31] = checkBoxA1_32;
            bitLevels[32] = checkBoxB1_33;
            bitLevels[33] = checkBoxB1_34;
            bitLevels[34] = checkBoxB1_35;
            bitLevels[35] = checkBoxB1_36;
            bitLevels[36] = checkBoxB1_37;
            bitLevels[37] = checkBoxB1_38;
            bitLevels[38] = checkBoxB1_39;
            bitLevels[39] = checkBoxB1_40;
            bitLevels[40] = checkBoxC1_41;
            bitLevels[41] = checkBoxC1_42;
            bitLevels[42] = checkBoxC1_43;
            bitLevels[43] = checkBoxC1_44;
            bitLevels[44] = checkBoxC1_45;
            bitLevels[45] = checkBoxC1_46;
            bitLevels[46] = checkBoxC1_47;
            bitLevels[47] = checkBoxC1_48;


            for (int i = 0; i < numPorts; i++)
            {
                pData[i] = 0xFF;
                bIsInput[i] = true;  // Default power up not Output hence Input Mode
            }

        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            // Called before Form is displayed Initialize resources:

            // Add the event and the event handler for the method that will 
            // process the timer event to the timer:
            myTimer.Tick += new EventHandler(TimerEventProcessor);

            // If you know the IP address/hostname at design time, you could just go right in to connecting and start.

            // Set the timer interval in miliseconds and start 55ms min resolution:
            myTimer.Interval = 100;
            myTimer.Enabled = false;
        }
        
        private void TimerEventProcessor(Object myObject, EventArgs myEventArgs)
        {
            // This is the method that runs when the timer event is raised:
            myTimer.Stop();
 
            UpdateGUIState();
           
            myTimer.Enabled = true;
        }
        
        private void UpdateGUIState()
        {           
            // Get data from board update display states:
            UInt32 Status;

            Status = AEWNet.AEW_DIO_ReadAll(uClientRef, ref pDataBytes, pData); 

            // Test each bit and set check state:
            for (int i = 0; i < numPins; i++)
            {
                if ((pData[i / 8] & (1 << (i % 8))) != 0)   // looks complicated but efficient
                    bitLevels[i].Checked = true;      // High Level
                else
                    bitLevels[i].Checked = false;     // Low Level
            }
        }
 
        private void btnToggle_Click(object sender, EventArgs e)
        {
            // Button Clicks to toggle the input output mode and display states:

            // We are assuming we started in input mode:

            // Need to stop the timer change board and GUI:
            if (bConnected) // if connected
            {
                myTimer.Stop();
            }
            
            Int32 i;
            Int32 iOutMask = 0x0000;
            int numBytes = 6;

            //Get button sender Tag (index)
            UInt32 iTagindex = 0;
            Button btnSender = sender as Button;
            if (btnSender != null)
            {
                iTagindex = Convert.ToUInt32(btnSender.Tag);
            }

            // Toggle local mode value and button display states:
            if (bIsInput[iTagindex])
            {
                bIsInput[iTagindex] = false;
                btnSender.Text = "(Output) Press for Input Mode";
            }
            else
            {
                bIsInput[iTagindex] = true;
                btnSender.Text = "(Input) Press for Output Mode";
            }

            // Recreate new Outmask for Configure:            
            for (i = 0; i < numPorts; i++)
            {
                if (bIsInput[i] == true)  // if output mode
                {
                    // USB-DIO96 its 12 bits in one Int16 or short  48 is 6 ports
                    iOutMask = iOutMask | (1 << i); // Int32 
                }
            }

            i = (int)iTagindex;

            ushort iusMask = (ushort)iOutMask;
            AEWNet.AEW_DIO_Configure(uClientRef, ref numBytes, pData, ref iusMask);

            // Restart timer thread:
            if (bConnected) // if connected
            {
                myTimer.Enabled = true;
            }
        }

        private void checkBox_Click(object sender, EventArgs e)
        {
            if (bConnected) // if connected
            {
                myTimer.Stop();
            }

            Int32 bitIndex;
            Byte bByte = 1;

            bitIndex = Convert.ToInt32((sender as CheckBox).Tag);

            // toggle
            if (((pData[bitIndex / 8]) & (1 << (bitIndex % 8))) != 0)
                bByte = 0;

            Status = AEWNet.AEW_DIO_Write1(uClientRef, bitIndex, bByte);  // byte used like a bool

            if (bConnected) // if connected
            {
                myTimer.Enabled = true;
            }
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            Int32 Index = 0;
            String strIP;

            strIP = textBoxHostIP.Text + Char.MinValue; // Add null terminator.
            Char[] charIP = new Char[32];
            charIP = strIP.ToCharArray();

            labelStatus.Text = "Attempting to Connect...";
            Application.DoEvents();
            uClientRef = AEWNet.AEW_Connect(charIP, Index);

           
            // Test pointer then last error:
            if (uClientRef == UIntPtr.Zero) // null 
            {
                int lastError = Marshal.GetLastWin32Error();  // 32 bit specific !!!
                labelStatus.Text = "Error " + lastError.ToString() + " on AEW_Connect()";
            }
            else
            {
                int numBytes = 14;
                byte[] IOConfig = new byte[14] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
                AEWNet.AEW_DEF_GetIOConfig(uClientRef, ref numBytes, IOConfig);
                for (int portIndex = 0; portIndex < numPorts; portIndex++)
                {
                    pData[portIndex] = IOConfig[portIndex+1];
                    if ((IOConfig[0] & (1 << portIndex)) == 0) // IOConfig[0] is "current input ports mask" (1==input)
                    {
                        bIsInput[portIndex] = false;  // state were going to start with
                        IoButtons[portIndex].Text = "(Output) Press for Input Mode";
                    }
                    else 
                    { 
                        bIsInput[portIndex] = true;
                        IoButtons[portIndex].Text = "(Input) Press for Output Mode";
                    }
                }

                labelStatus.Text = "Connected.";
                bConnected = true;
                myTimer.Start();
                myTimer.Enabled = true;

            }

        }

        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            myTimer.Stop(); 
            
            UInt32 Status;

            labelStatus.Text = "Disonnecting...";
            Application.DoEvents(); 
            Status = AEWNet.AEW_Disconnect(uClientRef);
            labelStatus.Text = "Disonnected!";
            bConnected = false;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            myTimer.Stop();

            UInt32 Status;
            Status = AEWNet.AEW_Disconnect(uClientRef);
        }

        private void ConfigurePowerOnDefault_Click(object sender, EventArgs e)
        {
            int l = 7;
            byte[] IOConfig = new byte[7] { 0, 0, 0, 0, 0, 0, 0 };
            // grab current I/O Group input vs output configuration
            for (int i = 0; i < numPorts; i++)
            {
                if (bIsInput[i] == true)  // if input mode
                {
                    IOConfig[0] = (byte)(IOConfig[0] | (1 << i));
                }
            }

            // Grab current high/low state for all bits
            for (int i = 0; i < numPins; i++)
            {
                if (bitLevels[i].Checked == true)
                    IOConfig[i/8 +1] |=(byte) (1<<(i%8));     
            }
            AEWNet.AEW_DEF_Configure(uClientRef, ref l, IOConfig);
        }
    }
}